// Maze.java
//
// ICS 23Project #1: Dark at the End of the Tunnel
// Coded by Alex Thornton Fall 2002
// Minor edits by Norman Jacobson January 2003

// The Maze interface defines the behavior of a maze object.  A maze is a
// two-dimensional grid of "positions," each with an (x,y) coordinate.
// The top left corner of the maze is (0,0).
// Between each adjacent pair of positions, there may or may not be a wall.
//
// You create a Maze by calling this static method:
//
//     Maze MazeFactory.createMaze(int width, int height)
//
// The starting position of the new maze will always be (0,0) and the
// ending position will always be (width-1, height-1).
//
// NOTE: You may not need to use all of the methods in this interface.

/**
 * Describes all applicable functions to Mazes.
 */
public interface Maze
{
	/**
	 * Queries the number of cells in a row of this maze.
	 * @return the positive integer number
	 */
	public int getWidth();

	/**
	 * Queries the number of cells in a column of this maze.
	 * @return the positive integer number
	 */
	public int getHeight();

	/**
	 * Queries the x-coordinate of the entrance cell of this maze.
	 * @return the non-negative integer number
	 */
	public int getStartXPosition();

	/**
	 * Queries the y-coordinate of the entrance cell of this maze.
	 * @return the non-negative integer number
	 */
	public int getStartYPosition();

	/**
	 * Queries the x-coordinate of the exit cell of this maze.
	 * @return the non-negative integer number
	 */
	public int getEndXPosition();

	/**
	 * Queries the y-coordinate of the exit cell of this maze.
	 * @return the non-negative integer number
	 */
	public int getEndYPosition();


	// IllegalArgumentException is a standard Java class, and is an
	// unchecked exception. if this exception is thrown, your
	// program has a bug that needs to be fixed!

	/**
	 * Queries whether there is no wall on the upper edge of the specified cell in this maze.
	 * It throws an <TT>IllegalArgumentException</TT> if
	 * parameters indicate an invalid <TT>(x,y)</TT> coordinate for this maze (i.e.
	 * <TT>x</TT> is negative or >= the width, <TT>y</TT> is negative or >= the height).
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 * @return the Bool value
	 *
	 */
	public boolean canMoveNorthFrom(int x, int y);

	/**
	 * Queries whether there is no wall on the bottom edge of the specified cell in this maze.
	 * It throws an <TT>IllegalArgumentException</TT> if
	 * parameters indicate an invalid <TT>(x,y)</TT> coordinate for this maze (i.e.
	 * <TT>x</TT> is negative or >= the width, <TT>y</TT> is negative or >= the height).
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 * @return the Bool value
	 *
	 */
	public boolean canMoveSouthFrom(int x, int y);

	/**
	 * Queries whether there is no wall on the right edge of the specified cell in this maze.
	 * It throws an <TT>IllegalArgumentException</TT> if
	 * parameters indicate an invalid <TT>(x,y)</TT> coordinate for this maze (i.e.
	 * <TT>x</TT> is negative or >= the width, <TT>y</TT> is negative or >= the height).
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 * @return the Bool value
	 *
	 */
	public boolean canMoveEastFrom(int x, int y);

	/**
	 * Queries whether there is no wall on the left edge of the specified cell in this maze.
	 * It throws an <TT>IllegalArgumentException</TT> if
	 * parameters indicate an invalid <TT>(x,y)</TT> coordinate for this maze (i.e.
	 * <TT>x</TT> is negative or >= the width, <TT>y</TT> is negative or >= the height).
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 * @return the Bool value
	 *
	 */
	public boolean canMoveWestFrom(int x, int y);

	/**
	 * Adds a wall on the upper edge of the specified cell in this maze.
	 * If the wall already exists, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void addWallNorthOf(int x, int y);

	/**
	 * Adds a wall on the bottom edge of the specified cell in this maze.
	 * If the wall already exists, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void addWallSouthOf(int x, int y);

	/**
	 * Adds a wall on the right edge of the specified cell in this maze.
	 * If the wall already exists, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void addWallEastOf(int x, int y);

	/**
	 * Adds a wall on the left edge of the specified cell in this maze.
	 * If the wall already exists, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void addWallWestOf(int x, int y);

	/**
	 * Removes a wall on the upper edge of the specified cell in this maze.
	 * If the wall is already missing, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void removeWallNorthOf(int x, int y);

	/**
	 * Removes a wall on the bottom edge of the specified cell in this maze.
	 * If the wall is already missing, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void removeWallSouthOf(int x, int y);

	/**
	 * Removes a wall on the right edge of the specified cell in this maze.
	 * If the wall is already missing, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void removeWallEastOf(int x, int y);

	/**
	 * Removes a wall on the left edge of the specified cell in this maze.
	 * If the wall is already missing, nothing happens.
	 * @param x the x-coordinate of the cell
	 * @param y the y-coordinate of the cell
	 */
	public void removeWallWestOf(int x, int y);
}
